@@ -1,6 +1,32 @@
-## (c) Simon N. Wood 2011-2014
-## Many of the following are simple wrappers for C functions, used largely 
-## for testing purposes
+## (c) Simon N. Wood 2011-2019
+## Many of the following are simple wrappers for C functions
+
+"%.%" <- function(a,b) {
+  tensor.prod.model.matrix(list(as.matrix(a),as.matrix(b)))
+}
+
+blas.thread.test <- function(n=1000,nt=4) {
+  inter <- interactive()
+  if (inter) prg <- txtProgressBar(min = 0, max = n, initial = 0,
+              char = "=",width = NA, title="Progress", style = 3)
+  for (i in 1:n) {
+    m <- sample(213:654,1)
+    p <- sample(3:153,1)
+    X <- matrix(runif(m*p),m,p)
+    er <- pqr(X,nt=nt) 
+    rr <- range(pqr.qy(er,pqr.R(er))-X[,er$pivot])
+    if (rr[1] < -1e-4||rr[2] > 1e-4) {
+      break;
+    }
+    if (inter) setTxtProgressBar(prg, i)
+  }
+  if (inter) close(prg)
+  if (rr[1] < -1e-4||rr[2] > 1e-4) {
+    cat("BLAS thread safety problem at iteration",i,"\n")
+  } else cat("No problem encountered in",i,"iterations\n")
+} ## blas.thread.test
+
+
 
 rmvn <- function(n,mu,V) {
 ## generate multivariate normal deviates. e.g.
@@ -130,7 +156,7 @@ mvn.ll <- function(y,X,beta,dbeta=NULL) {
 
 ## discretized covariate routines...
 
-XWXd <- function(X,w,k,ks,ts,dt,v,qc,nthreads=1,drop=NULL,ar.stop=-1,ar.row=-1,ar.w=-1) {
+XWXd <- function(X,w,k,ks,ts,dt,v,qc,nthreads=1,drop=NULL,ar.stop=-1,ar.row=-1,ar.w=-1,lt=NULL,rt=NULL) {
 ## Form X'WX given weights in w and X in compressed form in list X.
 ## each element of X is a (marginal) model submatrix. Full version 
 ## is given by X[[i]][k[,i],] (see below for summation convention).
@@ -143,16 +169,52 @@ XWXd <- function(X,w,k,ks,ts,dt,v,qc,nthreads=1,drop=NULL,ar.stop=-1,ar.row=-1,a
 ## constraints (if qc[i]>1), stored as a householder vector in v[[i]]. 
 ## check ts and k index start (assumed 1 here)
 ## if drop is non-NULL it contains index of rows/cols to drop from result
+## * lt is array of terms to include in left matrix (assumed in ascending coef index order)
+## * rt is array of terms to include in right matrix (assumed in ascending coef index order)
+## * if both NULL all are terms are included, if only one is NULL then used for left and right. 
   m <- unlist(lapply(X,nrow));p <- unlist(lapply(X,ncol))
   nx <- length(X);nt <- length(ts)
   n <- length(w);pt <- 0;
   for (i in 1:nt) pt <- pt + prod(p[ts[i]:(ts[i]+dt[i]-1)]) - as.numeric(qc[i]>0)
   ## block oriented code...
-  t0 <- system.time(oo <- .C(C_XWXd0,XWX =as.double(rep(0,(pt+nt)^2)),X= as.double(unlist(X)),w=as.double(w),
+  if (is.null(lt)&&is.null(lt)) {
+    #t0 <- system.time(
+    oo <- .C(C_XWXd0,XWX =as.double(rep(0,(pt+nt)^2)),X= as.double(unlist(X)),w=as.double(w),
            k=as.integer(k-1),ks=as.integer(ks-1),m=as.integer(m),p=as.integer(p), n=as.integer(n), 
            ns=as.integer(nx), ts=as.integer(ts-1), as.integer(dt), nt=as.integer(nt),
            v = as.double(unlist(v)),qc=as.integer(qc),nthreads=as.integer(nthreads),
-           ar.stop=as.integer(ar.stop-1),ar.row=as.integer(ar.row-1),ar.weights=as.double(ar.w)))
+           ar.stop=as.integer(ar.stop-1),ar.row=as.integer(ar.row-1),ar.weights=as.double(ar.w))#)
+    XWX <- if (is.null(drop)) matrix(oo$XWX[1:pt^2],pt,pt) else matrix(oo$XWX[1:pt^2],pt,pt)[-drop,-drop]
+  } else {
+    lpip <- attr(X,"lpip") ## list of coefs for each term
+    rpi <- unlist(lpip[rt])
+    lpi <- unlist(lpip[lt])
+    if (is.null(lt)) {
+      lpi <- rpi
+      nrs <- lt <- 0
+      ncs <- length(rt)
+    } else {
+      nrs <- length(lt)
+      if (is.null(rt)) {
+        rt <- ncs <- 0
+	rpi <- lpi
+      } else ncs <- length(rt)
+    }  
+    #t0 <- system.time(
+    oo <- .C(C_XWXd1,XWX =as.double(rep(0,(pt+nt)^2)),X= as.double(unlist(X)),w=as.double(w),
+           k=as.integer(k-1),ks=as.integer(ks-1),m=as.integer(m),p=as.integer(p), n=as.integer(n), 
+           ns=as.integer(nx), ts=as.integer(ts-1), as.integer(dt), nt=as.integer(nt),
+           v = as.double(unlist(v)),qc=as.integer(qc),nthreads=as.integer(nthreads),
+           ar.stop=as.integer(ar.stop-1),ar.row=as.integer(ar.row-1),ar.weights=as.double(ar.w),rs=as.integer(lt-1),
+	   cs=as.integer(rt-1),nrs=as.integer(nrs),ncs=as.integer(ncs))#)
+    XWX <- matrix(oo$XWX[1:(length(lpi)*length(rpi))],length(lpi),length(rpi))
+    if (!is.null(drop)) {
+      ldrop <- which(lpi %in% drop)
+      rdrop <- which(lpi %in% drop)
+      if (length(ldrop)>0||length(rdrop)>0) XWX <-
+        if (length(ldrop==0)) XWX[,-rdrop] else if (length(rdrop)==0) XWX[-ldrop,] else XWX[-ldrop,-rdrop]
+    }
+  }
 ## old strictly level 2 code for comparison...	   
 #  t1 <- system.time(ooo <- .C(C_XWXd,XWX =as.double(rep(0,pt^2)),X= as.double(unlist(X)),w=as.double(w),
 #           k=as.integer(k-1),ks=as.integer(ks-1),m=as.integer(m),p=as.integer(p), n=as.integer(n), 
@@ -163,26 +225,44 @@ XWXd <- function(X,w,k,ks,ts,dt,v,qc,nthreads=1,drop=NULL,ar.stop=-1,ar.row=-1,a
 #  XWX <- matrix(oo$XWX[1:pt^2],pt,pt)
 #  XWX0 <- matrix(ooo$XWX[1:pt^2],pt,pt)
 #  plot(XWX0,XWX,pch=".",main=range(XWX-XWX0));abline(0,1,col=2)
-  if (is.null(drop)) matrix(oo$XWX[1:pt^2],pt,pt) else matrix(oo$XWX[1:pt^2],pt,pt)[-drop,-drop]
+  #if (is.null(drop)) matrix(oo$XWX[1:pt^2],pt,pt) else matrix(oo$XWX[1:pt^2],pt,pt)[-drop,-drop]
+  XWX
 } ## XWXd
 
-XWyd <- function(X,w,y,k,ks,ts,dt,v,qc,drop=NULL,ar.stop=-1,ar.row=-1,ar.w=-1) {
-## X'Wy...  
+XWyd <- function(X,w,y,k,ks,ts,dt,v,qc,drop=NULL,ar.stop=-1,ar.row=-1,ar.w=-1,lt=NULL) {
+## X'Wy...
+## if lt if not NULL then it lists the discrete terms to include (from X)
+## returned vector/matrix only includes rows for selected terms
   m <- unlist(lapply(X,nrow));p <- unlist(lapply(X,ncol))
   nx <- length(X);nt <- length(ts)
-  n <- length(w);pt <- 0;
-  for (i in 1:nt) pt <- pt + prod(p[ts[i]:(ts[i]+dt[i]-1)]) - as.numeric(qc[i]>0) 
-  oo <- .C(C_XWyd,XWy=rep(0,pt),y=as.double(y),X=as.double(unlist(X)),w=as.double(w),k=as.integer(k-1), 
+  n <- length(w);##pt <- 0;
+  ##for (i in 1:nt) pt <- pt + prod(p[ts[i]:(ts[i]+dt[i]-1)]) - as.numeric(qc[i]>0)
+  if (is.null(lt)) {
+    pt <- 0
+    for (i in 1:nt) pt <- pt + prod(p[ts[i]:(ts[i]+dt[i]-1)]) - as.numeric(qc[i]>0)
+    lt <- 1:nt
+  } else {
+    lpip <- attr(X,"lpip") ## list of coefs for each term 
+    lpi <- unlist(lpip[lt]) ## coefs corresponding to terms selected by lt
+    if (!is.null(drop)) drop <- which(lpi %in% drop) ## rebase drop 
+    pt <- length(lpi)
+  }
+  cy <- if (is.matrix(y)) ncol(y) else 1
+  oo <- .C(C_XWyd,XWy=rep(0,pt*cy),y=as.double(y),X=as.double(unlist(X)),w=as.double(w),k=as.integer(k-1), 
            ks=as.integer(ks-1),
-           m=as.integer(m),p=as.integer(p),n=as.integer(n), nx=as.integer(nx), ts=as.integer(ts-1), 
+           m=as.integer(m),p=as.integer(p),n=as.integer(n),cy=as.integer(cy), nx=as.integer(nx), ts=as.integer(ts-1), 
            dt=as.integer(dt),nt=as.integer(nt),v=as.double(unlist(v)),qc=as.integer(qc),
-           ar.stop=as.integer(ar.stop-1),ar.row=as.integer(ar.row-1),ar.weights=as.double(ar.w))
-  if (is.null(drop)) oo$XWy else oo$XWy[-drop]
+           ar.stop=as.integer(ar.stop-1),ar.row=as.integer(ar.row-1),ar.weights=as.double(ar.w),
+	   cs=as.integer(lt-1),ncs=as.integer(length(lt)))
+  if (cy>1) XWy <- if (is.null(drop)) matrix(oo$XWy,pt,cy) else matrix(oo$XWy,pt,cy)[-drop,] else
+  XWy <- if (is.null(drop)) oo$XWy else oo$XWy[-drop] 
+  XWy
 } ## XWyd 
 
-Xbd <- function(X,beta,k,ks,ts,dt,v,qc,drop=NULL) {
+Xbd <- function(X,beta,k,ks,ts,dt,v,qc,drop=NULL,lt=NULL) {
 ## note that drop may contain the index of columns of X to drop before multiplying by beta.
 ## equivalently we can insert zero elements into beta in the appropriate places.
+## if lt if not NULL then it lists the discrete terms to include (from X)
   n <- if (is.matrix(k)) nrow(k) else length(k) ## number of data
   m <- unlist(lapply(X,nrow)) ## number of rows in each discrete model matrix
   p <- unlist(lapply(X,ncol)) ## number of cols in each discrete model matrix
@@ -193,16 +273,28 @@ Xbd <- function(X,beta,k,ks,ts,dt,v,qc,drop=NULL) {
     if (is.matrix(beta)) b[-drop,] <- beta else b[-drop] <- beta
     beta <- b
   }
+  if (is.null(lt)) {
+    lt <- 1:nt
+  }
+  lpip <- attr(X,"lpip")
+  if (!is.null(lpip)) { ## then X list may not be in coef order...
+    lpip <- unlist(lpip[lt])
+    beta <- if (is.matrix(beta)) beta[lpip,] else beta[lpip] ## select params required in correct order
+  }
   bc <- if (is.matrix(beta)) ncol(beta) else 1 ## number of columns in beta
   oo <- .C(C_Xbd,f=as.double(rep(0,n*bc)),beta=as.double(beta),X=as.double(unlist(X)),k=as.integer(k-1),
            ks = as.integer(ks-1), 
            m=as.integer(m),p=as.integer(p), n=as.integer(n), nx=as.integer(nx), ts=as.integer(ts-1), 
-           as.integer(dt), as.integer(nt),as.double(unlist(v)),as.integer(qc),as.integer(bc))
+           as.integer(dt), as.integer(nt),as.double(unlist(v)),as.integer(qc),as.integer(bc),as.integer(lt-1),as.integer(length(lt)))
   if (is.matrix(beta)) matrix(oo$f,n,bc) else oo$f
 } ## Xbd
 
-diagXVXd <- function(X,V,k,ks,ts,dt,v,qc,drop=NULL,n.threads=1) {
+diagXVXd <- function(X,V,k,ks,ts,dt,v,qc,drop=NULL,nthreads=1,lt=NULL,rt=NULL) {
 ## discrete computation of diag(XVX')
+## BUGS: 1. X list may not be in coefficient order - in which case V has to be re-ordered according to lpip
+##          attribute of X (if non-null)
+##       2. It doesn't seem to work if terms are dropped, not from end, although Xbd seems to work in these cases.
+
   n <- if (is.matrix(k)) nrow(k) else length(k)
   m <- unlist(lapply(X,nrow));p <- unlist(lapply(X,ncol))
   nx <- length(X);nt <- length(ts)
@@ -211,11 +303,19 @@ diagXVXd <- function(X,V,k,ks,ts,dt,v,qc,drop=NULL,n.threads=1) {
     V0 <- matrix(0,pv,pv)
     V0[-drop,-drop] <- V
     V <- V0;rm(V0)
-  } else pv <- ncol(V) 
+  } else pv <- ncol(V)
+  if (is.null(lt)) lt <- 1:nt
+  if (is.null(rt)) rt <- 1:nt
+  lpip <- attr(X,"lpip")
+  if (!is.null(lpip)) { ## then X list may not be in coef order...
+    lpi <- unlist(lpip[lt])
+    rpi <- unlist(lpip[rt])
+    V <- V[lpi,rpi] ## select part of V required required in correct order
+  }
   oo <- .C(C_diagXVXt,diag=as.double(rep(0,n)),V=as.double(V),X=as.double(unlist(X)),k=as.integer(k-1), 
-           ks=as.integer(ks-1),
-           m=as.integer(m),p=as.integer(p), n=as.integer(n), nx=as.integer(nx), ts=as.integer(ts-1), 
-           as.integer(dt), as.integer(nt),as.double(unlist(v)),as.integer(qc),as.integer(pv),as.integer(n.threads))
+           ks=as.integer(ks-1),m=as.integer(m),p=as.integer(p), n=as.integer(n), nx=as.integer(nx),
+	   ts=as.integer(ts-1), as.integer(dt), as.integer(nt),as.double(unlist(v)),as.integer(qc),as.integer(nrow(V)),as.integer(ncol(V)),
+	   as.integer(nthreads),as.integer(lt-1),as.integer(length(lt)),as.integer(rt-1),as.integer(length(rt)))
   oo$diag
 } ## diagXVXd
 
@@ -382,6 +482,7 @@ block.reorder <- function(x,n.blocks=1,reverse=FALSE) {
 pqr <- function(x,nt=1) {
 ## parallel QR decomposition, using openMP in C, and up to nt threads (only if worthwhile)
 ## library(mgcv);n <- 20;p<-4;X <- matrix(runif(n*p),n,p);er <- mgcv:::pqr(X,nt=2)
+## range(mgcv:::pqr.qy(er,mgcv:::pqr.R(er))-X[,er$pivot])
   x.c <- ncol(x);r <- nrow(x)
   oo <- .C(C_mgcv_pqr,x=as.double(c(x,rep(0,nt*x.c^2))),as.integer(r),as.integer(x.c),
            pivot=as.integer(rep(0,x.c)), tau=as.double(rep(0,(nt+1)*x.c)),as.integer(nt)) 
