@@ -429,6 +429,11 @@ smooth.construct.so.smooth.spec<-function(object,data,knots)
   bnd <- object$xt$bnd
   if (is.null(bnd)) stop("can't soap smooth without a boundary")
   if (!inherits(bnd,"list")) stop("bnd must be a list of boundary loops")
+
+  ## check knots within boundary...
+  kin <- in.out(bnd,cbind(knt[[1]],knt[[2]]))
+  if (any(!kin)) warning("dropping soap knots not inside boundary - use 'in.out' to investigate.")
+  knt[[1]] <- knt[[1]][kin];knt[[2]] <- knt[[2]][kin] 
   
   for (i in 1:length(bnd)) { ## re-lable boundary
     nm <- names(bnd[[i]])
@@ -482,6 +487,8 @@ smooth.construct.so.smooth.spec<-function(object,data,knots)
 
   object$irng <- irng ## the column scaling factor 
 
+  if (any(!is.finite(irng))) stop("soap basis ill-conditioned - changing 'xt$nmax' may help")
+
   object$X <- b$X ## model matrix
   attr(object$X,"offset") <- b$offset
 
@@ -745,7 +752,7 @@ Predict.matrix.sw <- function(object,data)
 }
 
 plot.soap.film <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
-                     partial.resids=FALSE,rug=TRUE,se=TRUE,scale=-1,n=100,n2=40,
+                     partial.resids=FALSE,rug=TRUE,se=TRUE,scale=-1,n=100,n2=40,n3=3,
                      pers=FALSE,theta=30,phi=30,jit=FALSE,xlab=NULL,ylab=NULL,main=NULL,
                      ylim=NULL,xlim=NULL,too.far=0.1,shade=FALSE,shade.col="gray80",
                      shift=0,trans=I,by.resids=FALSE,scheme=0,hcolors=heat.colors(100),
