@@ -194,14 +194,14 @@ void mgcv_pqr(double *x,int *r, int *c,int *pivot, double *tau, int *nt);
 void getRpqr(double *R,double *x,int *r, int *c,int *rr,int *nt);
 void mgcv_pqrqy(double *b,double *a,double *tau,int *r,int *c,int *cb,int *tp,int *nt);
 SEXP mgcv_Rpiqr(SEXP X, SEXP BETA,SEXP PIV,SEXP NT,SEXP NB);
-void mgcv_tmm(SEXP x,SEXP t,SEXP D,SEXP M, SEXP N);
-void mgcv_Rpbsi(SEXP A, SEXP NT);
-void mgcv_RPPt(SEXP a,SEXP r, SEXP NT);
+SEXP mgcv_tmm(SEXP x,SEXP t,SEXP D,SEXP M, SEXP N);
+SEXP mgcv_Rpbsi(SEXP A, SEXP NT);
+SEXP mgcv_RPPt(SEXP a,SEXP r, SEXP NT);
 SEXP mgcv_Rpchol(SEXP Amat,SEXP PIV,SEXP NT,SEXP NB);
 void dchol(double *dA, double *R, double *dR,int *p);
 void chol_down(double *R,double *Rup,int *n,int *k,int *ut);
-void mgcv_chol_down(SEXP r,SEXP ru,SEXP N,SEXP K, SEXP UT);
-void mgcv_chol_up(SEXP r,SEXP U,SEXP N,SEXP UP,SEXP EPS);
+SEXP mgcv_chol_down(SEXP r,SEXP ru,SEXP N,SEXP K, SEXP UT);
+SEXP mgcv_chol_up(SEXP r,SEXP U,SEXP N,SEXP UP,SEXP EPS);
 void vcorr(double *dR,double *Vr,double *Vb,int *p,int *M);
 SEXP mgcv_Rpforwardsolve(SEXP R, SEXP B,SEXP NT);
 SEXP mgcv_Rpbacksolve(SEXP R, SEXP B,SEXP NT);
